"""
Base class for reading samples.

@author: Norman MacDonald
@date: 2010-02-16
"""

import sys
from pica.Sample import Sample, SampleSet, ClassLabelSet
from pica.DataCollection import DataCollection

class FileIO():
	def __init__(self):
		pass

	def load_samples(self,filename):
		"""Load samples from a tab delimited file."""
		fin = open(filename)
		data = fin.readlines()
		fin.close()
		"First, get a list of all attributes"
		attribute_to_index = {}
		index_to_attribute = []
		nattributes = 0
		for row in data:
			fields = (x.strip() for x in row.split()[1:])
			for field in fields:
				if not attribute_to_index.has_key(field):
					attribute_to_index[field] = nattributes
					index_to_attribute.append(field)
					nattributes += 1
					
		max_attribute = nattributes - 1
		sample_set = SampleSet(max_attribute)
		
		sample_set.feature_to_index = attribute_to_index
		sample_set.index_to_feature = index_to_attribute
		
		samples = [] 
		for line in data:
			fields = [x.strip() for x in line.split()]
			who = fields[0]
			attributes = map(lambda f: sample_set.feature_to_index[f.strip()],fields[1:])
			sample_set.add_sample(who,attributes)
			
		return sample_set
		
		
	def save_samples(self,sample_set,filename,sep="\t"):
		fout = open(filename,"w")
		for sample in sample_set:
			outline = [sample.id]
			string_list = sample.get_attributes_string_list()
			string_list.sort()
			outline.extend(string_list)
			fout.write("%s\n"%(sep.join(outline)))
		fout.close()
		
	def load_classes(self,filename):
		"""Load class labels from a tab delimited file."""
		class_label_set = ClassLabelSet()
		return class_label_set.load_data_collection(filename)
		
	
	def load_metadata(self,filename):
		data_collection = DataCollection()
		data_collection = data_collection.load_data_collection(filename)
		return data_collection
	
	
	
